#!/usr/bin/env python
import unittest
from runtime import Runtime


class RuntimeTestCase(unittest.TestCase):
    def test_parallel_exec(self):
        ret = Runtime._parallel_exec(lambda x: x * 2, xrange(5), n_threads=20)
        self.assertEqual(ret.get(), [0, 2, 4, 6, 8])


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
"""
Test functions related to controlled command execution
"""

from __future__ import print_function

import unittest

import os
import tempfile
import shutil

import logging

import exectools


class RetryTestCase(unittest.TestCase):
    """
    Test the exectools.retry() method
    """
    ERROR_MSG = r"Giving up after {} failed attempt\(s\)"

    def test_success(self):
        """
        Given a function that passes, make sure it returns successfully with
        a single retry or greater.
        """
        pass_function = lambda: True
        self.assertTrue(exectools.retry(1, pass_function))
        self.assertTrue(exectools.retry(2, pass_function))

    def test_failure(self):
        """
        Given a function that fails, make sure that it raise an exception
        correctly with a single retry limit and greater.
        """
        fail_function = lambda: False
        self.assertRaisesRegexp(
            Exception, self.ERROR_MSG.format(1), exectools.retry, 1, fail_function)
        self.assertRaisesRegexp(
            Exception, self.ERROR_MSG.format(2), exectools.retry, 2, fail_function)

    def test_wait(self):
        """
        Verify that the retry fails and raises an exception as needed.
        Further, verify that the indicated wait loops occurred.
        """

        expected_calls = list("fw0fw1f")

        # initialize a collector for loop information
        calls = []

        # loop 3 times, writing into the collector each try and wait
        self.assertRaisesRegexp(
            Exception, self.ERROR_MSG.format(3),
            exectools.retry, 3, lambda: calls.append("f"),
            wait_f=lambda n: calls.extend(("w", str(n))))

        # check that the test and wait loop operated as expected
        self.assertEqual(calls, expected_calls)

    def test_return(self):
        """
        Verify that the retry task return value is passed back out faithfully.
        """
        obj = {}
        func = lambda: obj
        self.assertIs(exectools.retry(1, func, check_f=lambda _: True), obj)


class TestCmdExec(unittest.TestCase):
    """
    """

    def setUp(self):
        self.test_dir = tempfile.mkdtemp(prefix="ocp-cd-test-logs")
        self.test_file = os.path.join(self.test_dir, "test_file")
        logging.basicConfig(filename=self.test_file, level=logging.INFO)
        self.logger = logging.getLogger()

    def tearDown(self):
        logging.shutdown()
        reload(logging)
        shutil.rmtree(self.test_dir)

    def test_cmd_assert_success(self):
        """
        """

        try:
            exectools.cmd_assert("/bin/true")
        except IOError as error:
            self.Fail("/bin/truereturned failure: {}".format(error))

        # check that the log file has all of the tests.
        log_file = open(self.test_file, 'r')
        lines = log_file.readlines()
        log_file.close()

        self.assertEquals(len(lines), 4)

    def test_cmd_assert_fail(self):
        """
        """

        # Try a failing command 3 times, at 1 sec intervals
        with self.assertRaises(IOError):
            exectools.cmd_assert("/usr/bin/false", 3, 1)

        # check that the log file has all of the tests.
        log_file = open(self.test_file, 'r')
        lines = log_file.readlines()
        log_file.close()

        self.assertEquals(len(lines), 12)


class TestGather(unittest.TestCase):
    """
    """

    def setUp(self):
        self.test_dir = tempfile.mkdtemp(prefix="ocp-cd-test-logs")
        self.test_file = os.path.join(self.test_dir, "test_file")
        logging.basicConfig(filename=self.test_file, level=logging.INFO)
        self.logger = logging.getLogger()

    def tearDown(self):
        logging.shutdown()
        reload(logging)
        shutil.rmtree(self.test_dir)

    def test_gather_success(self):
        """
        """

        (status, stdout, stderr) = exectools.cmd_gather(
            "/usr/bin/echo hello there")
        status_expected = 0
        stdout_expected = "hello there\n"
        stderr_expected = ""

        self.assertEquals(status_expected, status)
        self.assertEquals(stdout, stdout_expected)
        self.assertEquals(stderr, stderr_expected)

        # check that the log file has all of the tests.

        log_file = open(self.test_file, 'r')
        lines = log_file.readlines()

        self.assertEquals(len(lines), 6)

    def test_gather_fail(self):
        """
        """

        (status, stdout, stderr) = exectools.cmd_gather(
            ["/usr/bin/sed", "-e", "f"])

        status_expected = 1
        stdout_expected = ""
        stderr_expected = "/usr/bin/sed: -e expression #1, char 1: unknown command: `f'\n"

        self.assertEquals(status_expected, status)
        self.assertEquals(stdout, stdout_expected)
        self.assertEquals(stderr, stderr_expected)

        # check that the log file has all of the tests.
        log_file = open(self.test_file, 'r')
        lines = log_file.readlines()

        self.assertEquals(len(lines), 6)


if __name__ == "__main__":

    unittest.main()



import shlex
import exectools
import logutil

logger = logutil.getLogger(__name__)


class DockerContainer(object):
    """This is an abstraction for launching containers from images and
then running a subset of operations in the new container."""

    def __init__(self, image, autostart=False):
        self.image = image
        self.name_tag = image.split('/')[-1]
        self.logger = logutil.EntityLoggingAdapter(logger, {'entity': '{}'.format(self.name_tag)})
        self.cid = None
        if autostart:
            self.start()

    def _cmd(self, cmd):
        """Run a docker related command"""
        return exectools.cmd_gather(shlex.split(cmd))

    def start(self):
        """Start the container"""
        run_str = 'docker run --network=host -d -ti -u 0 --entrypoint /bin/bash {img}'.format(img=self.image)
        self.logger.info("[Verify] image: {img}".format(img=self.name_tag))
        # TODO: Check if this failed
        rc, stdout, stderr = self._cmd(run_str)
        self.cid = stdout.rstrip()

    def stop(self):
        """Stop the container"""
        return self._cmd('docker stop {cid}'.format(cid=self.cid))

    def rm(self):
        """Erase the used container"""
        return self._cmd('docker rm {cid}'.format(cid=self.cid))

    def execute(self, cmd):
        """Execute a command in the container

        :param string cmd: The command to run the the container. This
        will be automatically split into a shell acceptable format
        using the shlex library. Fancy/tedious escaping of quote
        characters is not necessary.

        """
        return self._cmd('docker exec {cid} {cmd}'.format(cid=self.cid, cmd=cmd))

    def copy_into(self, source, dest):
        """Copy a file into the container

        :param string source: The fully qualified path to the file to
        copy into the container

        :param string dest: The destination directory to copy the
        `source` file into in the container

        """
        return self._cmd('docker cp {source} {cid}:{dest}'.format(source=source, cid=self.cid, dest=dest))



#!/usr/bin/env python
"""
Test Objects to manage Docker containers.
"""

import unittest

# Helper modules for testing
import string
import StringIO
import re
import logging

# The test module
import container


class TestDockerContainer(unittest.TestCase):
    """
    Test docker container object and methods
    """

    def setUp(self):
        """
        Define and provide mock logging for test/response
        """
        self.stream = StringIO.StringIO()
        logging.basicConfig(level=logging.DEBUG, stream=self.stream)
        self.logger = logging.getLogger()

    def tearDown(self):
        """
        Reset logging for each test.
        """
        logging.shutdown()
        reload(logging)

    def test_cmd_logging(self):
        """
        Test the internal wrapper function for exectools.cmd_gather().
        This method executes a command, logs both the command and stdout
        to the same log destination.

        For testing, just execute an echo and gather the output.
        """

        # The first two lines are RE patterns because the log entries
        # will contain the CWD path.
        expected = [
            "INFO:root:Executing:cmd_gather \[[^\]]+\]: \['echo', 'hello'\]",
            "INFO:root:Process \[[^\]]+\]: \['echo', 'hello'\]: exited with: 0",
            "stdout>>hello",
            "<<",
            "stderr>><<",
            "",
            ""
        ]

        c0 = container.DockerContainer("test/image")
        c0._cmd("echo hello")

        actual = self.stream.getvalue()
        lines = string.split(actual, "\n")
        self.assertEqual(len(lines), 7)

        # check that the first and second lines match the expected patterns.
        self.assertTrue(
            re.match(expected[0], lines[0]),
            "process exit line does not match: \n  actual: {}\n  expected {}".
            format(expected[1], lines[1])
        )
        self.assertTrue(
            re.match(expected[1], lines[1]),
            "process exit line does not match: \n  actual: {}\n  expected {}".
            format(expected[1], lines[1])
        )

        # The remainder of the output must match verbatim
        self.assertListEqual(lines[2:], expected[2:])

if __name__ == "__main__":
    unittest.main()



import metadata
import yaml
from pykwalify.core import Core
import os
import shutil
from pushd import Dir
import exectools
import sys


VALID_UPDATES = {
    'mode': metadata.CONFIG_MODES,
}


# Used in oit.py to print out valid update options
# in --help output
def valid_updates():
    res = '\n\tKey\tValid Options\n\n'
    for k, v in VALID_UPDATES.iteritems():
        opts = ""
        if v:
            v = [str(i) for i in v]
            opts = ':\t{}'.format(','.join(v))
        res += '\t{}{}\n\n'.format(k, opts)
    return res


class MetaDataConfig(object):
    """
    Holds common functions for managing the MetaData configs
    Mostly is a class to hold runtime
    """
    def __init__(self, runtime):
        self.runtime = runtime
        if self.runtime.remove_tmp_working_dir:
            print('config:* options require a non-temporary working space. Must run with --working-dir')
            sys.exit(1)

    def _do_update(self, meta, k, v):
        """
        Convenience function for setting meta keys
        """
        self.runtime.logger.info('{}: [{}] -> {}'.format(meta.config_filename, k, v))
        meta.config[k] = v
        meta.save()

    def update(self, key, val):
        """
        Update [key] to [val] in all given image/rpm metas
        VALID_UPDATES is used to lock out what can be updated
        Right now only [mode] is valid, but that may change
        """
        if key not in VALID_UPDATES:
            raise ValueError('{} is not a valid update key. See --help'.format(key))

        if VALID_UPDATES[key]:
            if val not in VALID_UPDATES[key]:
                msg = '{} is not a valid value for {}. Use one of: {}'.format(val, key, ','.join(VALID_UPDATES[key]))
                raise ValueError(msg)

        for img in self.runtime.image_metas():
            self._do_update(img, key, val)

        for rpm in self.runtime.rpm_metas():
            self._do_update(rpm, key, val)

    def config_print(self, key=None, name_only=False):
        """
        Print name, sub-key, or entire config
        """
        def _do_print(meta, k):
            if name_only:
                print(meta.config_filename)
            else:
                if k:
                    val = meta.config.get(k, None)
                else:
                    val = meta.config.primitive()

                val = yaml.safe_dump(val, default_flow_style=False)

                print("*****" + meta.config_filename + "*****")
                print(val)
                print('')

        image_metas = self.runtime.image_metas()
        rpm_metas = self.runtime.rpm_metas()

        if image_metas:
            print('')
            print('********* Images *********')
            for img in image_metas:
                _do_print(img, key)

        if rpm_metas:
            print('')
            print('*********  RPMs  *********')
            for rpm in rpm_metas:
                _do_print(rpm, key)

    def commit(self, msg):
        """
        Commit outstanding metadata config changes
        """
        self.gitdata.commit()

    def push(self):
        """
        Push changes back to config repo.
        Will of course fail if user does not have write access.
        """
        self.gitdata.push()



"""
Test the task related functions for the OpenShift Image/RPM Build Tool
"""
import unittest

import assertion


class TestAssert(unittest.TestCase):
    """
    Test the methods of the assertion module.

    Each raises an exception if the asserted test fails.
    """
    def test_isdir(self):
        """
        Verify both positive and negative results for directory test
        """
        dir_exists = "/usr"
        dir_missing = "/tmp/doesnotexist"
        not_dir = "/etc/motd"

        try:
            assertion.isdir(dir_exists, "dir missing: {}".format(dir_exists))
        except assertion.FileNotFoundError as fnf_error:
            self.fail("asserted real directory does not exist: {}".
                      format(fnf_error))

        with self.assertRaises(assertion.FileNotFoundError):
            assertion.isdir(dir_missing, "dir missing: {}".format(dir_missing))

        # This should raise NotADirectory
        with self.assertRaises(assertion.FileNotFoundError):
            assertion.isdir(not_dir, "file, not dir: {}".format(not_dir))

    def test_isfile(self):
        """
        Verify both positive and negative results for file test
        """
        file_exists = "/etc/motd"
        file_missing = "/tmp/doesnotexist"
        not_file = "/usr"

        try:
            assertion.isfile(file_exists, "file missing: {}".format(file_exists))
        except assertion.FileNotFoundError as fnf_error:
            self.fail("asserted real file does not exist: {}".format(fnf_error))

        with self.assertRaises(assertion.FileNotFoundError):
            assertion.isfile(
                file_missing,
                "file missing: {}".format(file_missing)
            )

        # Should raise IsADirectory
        with self.assertRaises(assertion.FileNotFoundError):
            assertion.isfile(not_file, "dir, not file: {}".format(not_file))

    def test_success(self):
        """
        Verify that return codes indicating pass and fail respond correctly
        """

        # When we move to Python 3 this will raise ChildProcessError directly
        try:
            assertion.success(0, "this should not fail")
        except assertion.ChildProcessError as proc_fail:
            self.fail("success reported as fail: {}".format(proc_fail))

        with self.assertRaises(assertion.ChildProcessError):
            assertion.success(1, "process failed")


if __name__ == "__main__":
    unittest.main()



from .runtime import Runtime
from .pushd import Dir


def version():
    from os.path import abspath, dirname, join
    filename = join(dirname(abspath(__file__)), 'VERSION')
    return open(filename).read().strip()



"""
This module contains a set of functions for managing shell commands
consistently. It adds some logging and some additional capabilties to the
ordinary subprocess behaviors.
"""

from __future__ import print_function

import subprocess
import time
import shlex
import os

from fcntl import fcntl, F_GETFL, F_SETFL
from os import O_NONBLOCK, read

import logutil
import pushd
import assertion
from util import red_print, green_print, yellow_print

SUCCESS = 0

logger = logutil.getLogger(__name__)


class RetryException(Exception):
    """
    Provide a custom exception for retry failures
    """
    pass


def retry(retries, task_f, check_f=bool, wait_f=None):
    """
    Try a function up to n times.
    Raise an exception if it does not pass in time

    :param retries int: The number of times to retry
    :param task_f func: The function to be run and observed
    :param func()bool check_f: a function to check if task_f is complete
    :param func()bool wait_f: a function to run between checks
    """
    for attempt in range(retries):
        ret = task_f()
        if check_f(ret):
            return ret
        if attempt < retries - 1 and wait_f is not None:
            wait_f(attempt)
    raise RetryException("Giving up after {} failed attempt(s)".format(retries))


def cmd_assert(cmd, retries=1, pollrate=60, on_retry=None, set_env=None):
    """
    Run a command, logging (using exec_cmd) and raise an exception if the
    return code of the command indicates failure.
    Try the command multiple times if requested.

    :param cmd <string|list>: A shell command
    :param retries int: The number of times to try before declaring failure
    :param pollrate int: how long to sleep between tries
    :param on_retry <string|list>: A shell command to run before retrying a failure
    :param set_env: Dict of env vars to set for command (overriding existing)
    :return: (stdout,stderr) if exit code is zero
    """

    for try_num in range(0, retries):
        if try_num > 0:
            logger.debug(
                "cmd_assert: Failed {} times. Retrying in {} seconds: {}".
                format(try_num, pollrate, cmd))
            time.sleep(pollrate)
            if on_retry is not None:
                cmd_gather(on_retry, set_env)  # no real use for the result though

        result, stdout, stderr = cmd_gather(cmd, set_env)
        if result == SUCCESS:
            break

    logger.debug("cmd_assert: Final result = {} in {} tries.".format(result, try_num))

    assertion.success(
        result,
        "Error running [{}] {}. See debug log.".
        format(pushd.Dir.getcwd(), cmd))

    return stdout, stderr


def cmd_gather(cmd, set_env=None, realtime=False):
    """
    Runs a command and returns rc,stdout,stderr as a tuple.

    If called while the `Dir` context manager is in effect, guarantees that the
    process is executed in that directory, even if it is no longer the current
    directory of the process (i.e. it is thread-safe).

    :param cmd: The command and arguments to execute
    :param set_env: Dict of env vars to set for command (overriding existing)
    :param realtime: If True, output stdout and stderr in realtime instead of all at once.
    :return: (rc,stdout,stderr)
    """

    if not isinstance(cmd, list):
        cmd_list = shlex.split(cmd)
    else:
        cmd_list = cmd

    cwd = pushd.Dir.getcwd()
    cmd_info = '[cwd={}]: {}'.format(cwd, cmd_list)

    env = os.environ.copy()
    if set_env:
        cmd_info = '[env={}] {}'.format(set_env, cmd_info)
        env.update(set_env)
    logger.debug("Executing:cmd_gather {}".format(cmd_info))
    try:
        proc = subprocess.Popen(
            cmd_list, cwd=cwd, env=env,
            stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    except OSError as exc:
        logger.error("Subprocess errored running:\n{}\nWith error:\n{}\nIs {} installed?".format(
            cmd_info, exc, cmd_list[0]
        ))
        return exc.errno, "", "See previous error description."

    if not realtime:
        out, err = proc.communicate()
        rc = proc.returncode
    else:
        out = ""
        err = ""

        # Many thanks to http://eyalarubas.com/python-subproc-nonblock.html
        # setup non-blocking read
        # set the O_NONBLOCK flag of proc.stdout file descriptor:
        flags = fcntl(proc.stdout, F_GETFL)  # get current proc.stdout flags
        fcntl(proc.stdout, F_SETFL, flags | O_NONBLOCK)
        # set the O_NONBLOCK flag of proc.stderr file descriptor:
        flags = fcntl(proc.stderr, F_GETFL)  # get current proc.stderr flags
        fcntl(proc.stderr, F_SETFL, flags | O_NONBLOCK)

        rc = None
        while rc is None:
            output = None
            try:
                output = read(proc.stdout.fileno(), 256)
                green_print(output.rstrip())
                out += output
            except OSError:
                pass

            error = None
            try:
                error = read(proc.stderr.fileno(), 256)
                yellow_print(error.rstrip())
                out += error
            except OSError:
                pass

            rc = proc.poll()

    logger.debug(
        "Process {}: exited with: {}\nstdout>>{}<<\nstderr>>{}<<\n".
        format(cmd_info, rc, out, err))
    return rc, out, err




"""
Test the Dir() class.  Verify that it works as a context_manager.
"""

import unittest

import os
from multiprocessing.dummy import Pool

import pushd


class DirTestCase(unittest.TestCase):
    """
    Test the features of the pushd.Dir() class.  This is a context manager.
    It is meant to be used to allow a process to work in different working
    directories and return cleanly to the original working directory.

    It also locks so that the directory can only be visited by one thread at a
    time.
    """

    def test_chdir(self):
        """
        Verify that when a Dir is created and used in a `with` context, the
        CWD changes from current to new and back after exiting the context
        """
        cwd = os.getcwd()
        with pushd.Dir("/"):
            self.assertEqual(os.getcwd(), "/")
            with pushd.Dir("/dev"):
                self.assertEqual(os.getcwd(), "/dev")
            self.assertEqual(os.getcwd(), "/")
        self.assertEqual(os.getcwd(), cwd)

    def test_getcwd(self, concurrent=False):
        """
        Verify that the directory locking for concurrency is working
        """
        cwd = os.getcwd()
        if not concurrent:
            self.assertEqual(pushd.Dir.getcwd(), cwd)
        else:
            # the initial value is not reliable when using multiple threads
            cwd = pushd.Dir.getcwd()
        with pushd.Dir("/"):
            self.assertEqual(pushd.Dir.getcwd(), "/")
            with pushd.Dir("/dev"):
                self.assertEqual(pushd.Dir.getcwd(), "/dev")
            self.assertEqual(pushd.Dir.getcwd(), "/")
        self.assertEqual(pushd.Dir.getcwd(), cwd)

    def test_getcwd_threads(self):
        """
        Execute the concurrency test for 10 threads
        """
        thread_count = 10
        pool = Pool(thread_count)
        results = [
            pool.apply_async(lambda: self.test_getcwd(concurrent=True))
            for _ in range(thread_count)
        ]
        for result in results:
            result.get()


if __name__ == "__main__":
    unittest.main()
